#ifndef LCD_MODUL_H
#define LCD_MODUL_H
#include <string>
#include <thread>
#include <mutex>
#include <lcd.h>

using namespace std;

enum class Mode { fixed, flowing, effect, blinking, typed };

class LCD
{
    int fd;
    Mode mode[2] { Mode::fixed, Mode::fixed };
    void displayFixedText(int line, const string& text);
    void displayFlowingText(int line, const string& text);
    void displayEffectText(int line, const string& text);
    void displayBlinkingText(int line, const string& text);
    void displayTypedText(const string& text1, const string& text2);
    inline void displayChar(int line, int pos, const u_char c);
    bool stopDisplaying[2] {false, false};
    mutex uniqueAccessToLcdWrite;
    mutex blockLine[2];
    int velocity {40};
public:
    LCD();
    ~LCD();
    void clear();
    void display(int line, const string& text, const string& text2 = "");
    void setMode1(Mode mode) { this->mode[0] = mode; }
    void setMode2(Mode mode) { this->mode[1] = mode; }
    void setVelocity(int velocity) { this->velocity = velocity; }
    static const char COPYRIGHT {6};
    static const char SZLIG {7}; // ß
    static const char SMILE {8};
    static const char SAD {9};
    static const char HEART {10};
    static const char ARROW_LEFT {127};
    static const char ARROW_RIGHT {126};
    static const char UPPER_A_UMLAUT {11};
    static const char LOWER_A_UMLAUT {225};
    static const char O_UMLAUT {239};
    static const char U_UMLAUT {245};
    static const char DEGREE {223};
    static const char MY {228};
};

#endif // LCD_MODUL_H
