#include "widget.h"
#include "ui_widget.h"
#include <iostream>

using namespace std;

Widget::Widget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Widget)
{
    ui->setupUi(this);
}

Widget::~Widget()
{
    delete ui;
}

static void cutOversizedText(QTextEdit* textEdit, QLabel* label, QString& fixedText)
{
    if (!fixedText.isEmpty()) {
        textEdit->setHtml(fixedText);
        label->setText("0");
    }
}

void Widget::on_radioButtonFixed1_clicked()
{
    lcd.setMode1(Mode::fixed);
    ui->label1->setVisible(true);
    cutOversizedText(ui->textEdit1, ui->label1, fixedTexts[0]);
}

void Widget::on_radioButtonFlowing1_clicked()
{
    lcd.setMode1(Mode::flowing);
    ui->label1->setVisible(false);
}

void Widget::on_radioButtonEffect1_clicked()
{
    lcd.setMode1(Mode::effect);
    ui->label1->setVisible(true);
    cutOversizedText(ui->textEdit1, ui->label1, fixedTexts[0]);
}

void Widget::on_radioButtonBlinking1_clicked()
{
    lcd.setMode1(Mode::blinking);
    ui->label1->setVisible(true);
    cutOversizedText(ui->textEdit1, ui->label1, fixedTexts[0]);
}

void Widget::on_radioButtonFixed2_clicked()
{
    lcd.setMode2(Mode::fixed);
    ui->label2->setVisible(true);
    cutOversizedText(ui->textEdit2, ui->label2, fixedTexts[1]);
}

void Widget::on_radioButtonFlowing2_clicked()
{
    lcd.setMode2(Mode::flowing);
    ui->label2->setVisible(false);
}

void Widget::on_radioButtonEffect2_clicked()
{
    lcd.setMode2(Mode::effect);
    ui->label2->setVisible(true);
    cutOversizedText(ui->textEdit2, ui->label2, fixedTexts[1]);
}

void Widget::on_radioButtonBlinking2_clicked()
{
    lcd.setMode2(Mode::blinking);
    ui->label2->setVisible(true);
    cutOversizedText(ui->textEdit2, ui->label2, fixedTexts[1]);
}

void Widget::on_radioButtonTyped_clicked(bool checked)
{
    if (!checked) {
        ui->radioButtonTyped->setChecked(true);
        return;
    }
    lcd.setMode1(Mode::typed);
    lcd.setMode2(Mode::typed);
    ui->radioButtonTypedHidden1->setCheckable(true);
    ui->radioButtonTypedHidden2->setCheckable(true);
    ui->radioButtonTypedHidden1->setChecked(true);
    ui->radioButtonTypedHidden2->setChecked(true);
    ui->label1->setVisible(true);
    ui->label2->setVisible(true);
    cutOversizedText(ui->textEdit1, ui->label1, fixedTexts[0]);
    cutOversizedText(ui->textEdit2, ui->label2, fixedTexts[1]);
}

void Widget::on_radioButtonTypedHidden1_toggled(bool checked)
{
    if (!checked && ui->radioButtonTyped->isChecked()) {
        ui->radioButtonTypedHidden1->setCheckable(false);
        ui->radioButtonTypedHidden2->setCheckable(false);
        ui->radioButtonTyped->setChecked(false);
        ui->radioButtonFixed2->setChecked(true);
        on_radioButtonFixed2_clicked();
    }
}

void Widget::on_radioButtonTypedHidden2_toggled(bool checked)
{
    if (!checked && ui->radioButtonTyped->isChecked()) {
        ui->radioButtonTypedHidden1->setCheckable(false);
        ui->radioButtonTypedHidden2->setCheckable(false);
        ui->radioButtonTyped->setChecked(false);
        ui->radioButtonFixed1->setChecked(true);
        on_radioButtonFixed1_clicked();
    }
}

void Widget::on_sliderVelocity_valueChanged(int value)
{
    lcd.setVelocity(value);
}

static QString nonPrintableChars[] {"§", "\\", "~", "€",
    "á", "à", "â", "é", "è", "ê", "í", "ì", "î", "ó", "ò", "ô", "ú", "ù", "û",
    "ń", "ǹ", "ḿ", "ź", "ẑ", "ẃ", "ẁ", "ŵ"};
inline static bool containsAnyNonPrintableChar(const QString& s)
{
    for (const QString c: nonPrintableChars) {
        if (s.contains(c))
            return true;
    }
    return false;
}

const static QRegularExpression regexpGetInputForEnter {".+<body\\s.+?>\\s<p\\s.+?>(.*?)</p>",
                                                        QRegularExpression::MultilineOption | QRegularExpression::DotMatchesEverythingOption};
const static QString noInputIndicator {"-qt-paragraph-type:empty; "};

void Widget::handleEnterAndCheckInput(QTextEdit* textEdit, bool restrictInputSize, QLabel* label, QString& fixedText)
{
    QString plainText {textEdit->toPlainText()};
    if (plainText.contains('\n')) {
        QString html = textEdit->toHtml();
        html = html.indexOf(noInputIndicator) >= 0 ? regexpGetInputForEnter.match(html).captured(1) : "";
        textEdit->setHtml(html);
        on_showButton_clicked();
        return;
    }
    //Entferne Zeichen, die nicht angezeigt werden können.
    if (containsAnyNonPrintableChar(plainText)) {
        QString html = textEdit->toHtml();
        for (const QString c: nonPrintableChars)
            html.remove(c);
        textEdit->setHtml(html);
        return;
    }
    int textSize {plainText.size()};
    if (textSize == 16)
        fixedText = textEdit->toHtml();
    else if (textSize > 16) {
        if (restrictInputSize)
            textEdit->setHtml(fixedText);
    }
    else
        fixedText.clear();
    if (restrictInputSize)
        label->setText(textSize > 16 ? "0" : QString::number(16 - textSize));
}

void Widget::on_textEdit1_textChanged()
{
    handleEnterAndCheckInput(ui->textEdit1, !ui->radioButtonFlowing1->isChecked(), ui->label1, fixedTexts[0]);
}

void Widget::on_textEdit2_textChanged()
{
    handleEnterAndCheckInput(ui->textEdit2, !ui->radioButtonFlowing2->isChecked(), ui->label2, fixedTexts[1]);
}

static QString preIconTag {"<img src=\":/icons/"};
static QString postIconTag {".svg\" width=\"17\" height=\"17\" />"};
struct HtmlReplacement {QString html; QChar qchar;};
static HtmlReplacement iconReplacements[] {
    {preIconTag + "smiley" + postIconTag, LCD::SMILE},
    {preIconTag + "sad" + postIconTag, LCD::SAD},
    {preIconTag + "copyright" + postIconTag, LCD::COPYRIGHT},
    {preIconTag + "heart" + postIconTag, LCD::HEART},
    {preIconTag + "arrow_left" + postIconTag, LCD::ARROW_LEFT},
    {preIconTag + "arrow_right" + postIconTag, LCD::ARROW_RIGHT}
};
static HtmlReplacement htmlSpecialCharsReplacements[] {
    {"&lt;", '<'},
    {"&gt;", '>'},
    {"&quot;", '"'},
    {"&amp;", '&'}
};

const static QRegularExpression regexpGetInput {".+<body\\s.+?>\\s<p\\s.+?>(.*)</p></body></html>$",
                                                QRegularExpression::MultilineOption};

static const string& changeHtmlIntoLCDString(QString html) {
    static string result;
    result.clear();
    //Keine Eingabe?
    if (html.indexOf(noInputIndicator) >= 0)
        return result;
    html = regexpGetInput.match(html).captured(1);
    //Ersetze die Icons durch die Chars des LCDs.
    for (int i {0}; i != sizeof(iconReplacements)/sizeof(HtmlReplacement); ++i) {
        int j {0};
        while ((j = html.indexOf(iconReplacements[i].html)) != -1) {
            html.replace(j, iconReplacements[i].html.size(), iconReplacements[i].qchar);
        }
    }
    //Ersetze HTML kodierte Zeichen wie <, > usw.
    for (int i {0}; i != sizeof(htmlSpecialCharsReplacements)/sizeof(HtmlReplacement); ++i) {
        int j {0};
        while ((j = html.indexOf(htmlSpecialCharsReplacements[i].html)) != -1) {
            html.replace(j, htmlSpecialCharsReplacements[i].html.size(), htmlSpecialCharsReplacements[i].qchar);
        }
    }
    //Ersetze die Sonderzeichen.
    for (QChar qc: html) {
        if (qc == 228) {//ä
            result += LCD::LOWER_A_UMLAUT;
            continue;
        }
        if (qc == 196) {//Ä
            result += LCD::UPPER_A_UMLAUT;
            continue;
        }
        if (qc == 246 || qc == 214) {//ö, Ö
            result += LCD::O_UMLAUT;
            continue;
        }
        if (qc == 252 || qc == 220) {//ü, Ü
            result += LCD::U_UMLAUT;
            continue;
        }
        if (qc == 223) {//ß
            result += LCD::SZLIG;
            continue;
        }
        if (qc == 176) {//°
            result += LCD::DEGREE;
            continue;
        }
        if (qc == 181) {//µ
            result += LCD::MY;
            continue;
        }
        result += qc.toLatin1();
    }
    return result;
}

void Widget::on_showButton_clicked()
{
    if (ui->radioButtonTyped->isChecked()) {
        const string text2 {changeHtmlIntoLCDString(ui->textEdit2->toHtml())};
        lcd.display(0, changeHtmlIntoLCDString(ui->textEdit1->toHtml()),
                    text2);
    }
    else  {
        lcd.display(0, changeHtmlIntoLCDString(ui->textEdit1->toHtml()));
        lcd.display(1, changeHtmlIntoLCDString(ui->textEdit2->toHtml()));
    }
}

void Widget::addIcon(int line, Icon icon)
{
    QTextEdit* textEdit {(line == 0 ? ui->textEdit1 : ui->textEdit2)};
    textEdit->insertHtml(iconReplacements[(int) icon].html);
    textEdit->setFocus();
}

void Widget::on_pushButtonSmiley1_clicked()
{
    addIcon(0, Icon::SMILEY);
}

void Widget::on_pushButtonSad1_clicked()
{
    addIcon(0, Icon::SAD);
}

void Widget::on_pushButtonCopyright1_clicked()
{
    addIcon(0, Icon::COPYRIGHT);
}

void Widget::on_pushButtonHeart1_clicked()
{
    addIcon(0, Icon::HEART);
}

void Widget::on_pushButtonArrowLeft1_clicked()
{
    addIcon(0, Icon::ARROW_LEFT);
}

void Widget::on_pushButtonArrowRight1_clicked()
{
    addIcon(0, Icon::ARROW_RIGHT);
}

void Widget::on_pushButtonSmiley2_clicked()
{
    addIcon(1, Icon::SMILEY);
}

void Widget::on_pushButtonSad2_clicked()
{
    addIcon(1, Icon::SAD);
}

void Widget::on_pushButtonCopyright2_clicked()
{
    addIcon(1, Icon::COPYRIGHT);
}

void Widget::on_pushButtonHeart2_clicked()
{
    addIcon(1, Icon::HEART);
}

void Widget::on_pushButtonArrowLeft2_clicked()
{
    addIcon(1, Icon::ARROW_LEFT);
}

void Widget::on_pushButtonArrowRight2_clicked()
{
    addIcon(1, Icon::ARROW_RIGHT);
}
